<?php

namespace Drupal\qtools_profiler\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\qtools_common\Utility\QtoolsDebug;
use Drupal\qtools_profiler\PerformanceService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for resources preview report.
 */
class DrupalPreviewController extends ControllerBase {

  const HOOK_DRUPAL_RESOURCES_PREVIEW_ALTER = 'qtools_profiler_drupal_resources_preview';

  /**
   * {@inheritdoc}
   */
  public function __construct(
    protected PerformanceService $performanceService,
    protected AccountProxyInterface $accountProxy,
    protected RequestStack $requestStack,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get(PerformanceService::class),
      $container->get('current_user'),
      $container->get('request_stack'),
    );
  }

  /**
   * Returns page content.
   */
  public function previewResources(Request $request, $id): array {
    $record = $this->performanceService->getRequestRecord($id);
    if (empty($record)) {
      return [];
    }

    $session = $this->requestStack->getMainRequest()->getSession();
    $messages = $session->get('qpm', []);
    foreach ($messages as $key => $message) {
      if ($message[0] == $id) {
        $build[$key] = [
          '#type' => 'inline_template',
          '#template' => '<span style="color: green">{{ title }}</span>{{ content }}<span style="color: green">{{ caller }}</span><br />',
          '#context' => [
            'content' => QtoolsDebug::export(QtoolsDebug::cloneFormVarDumper($message[1])),
            'title' => $message[2],
            'caller' => $message[3],
          ],
        ];
      }
    }

    $build['record'] = [
      '#type' => 'inline_template',
      '#template' => '<h4>{{ title }}</h4>{{ content }}<br />',
      '#context' => [
        'title' => $this->t('Route key: @route', ['@route' => $record->route]),
        'content' => QtoolsDebug::export(QtoolsDebug::cloneFormVarDumper($record)),
      ],
    ];

    // Load and print customer info.
    $account = $this->accountProxy->getAccount();
    $export_data = [
      'current_user_account' => [
        $account,
        $this->t('Current user'),
      ],
    ];
    if ($account->isAuthenticated()) {
      $export_data['current_user_entity'] = [
        $this->entityTypeManager()->getStorage('user')->load($account->id()),
        $this->t('User entity'),
      ];
    }

    // Allow other modules to add state info without adding additional routes.
    $this->moduleHandler()->alter(static::HOOK_DRUPAL_RESOURCES_PREVIEW_ALTER, $export_data);

    foreach ($export_data as $key => $data) {
      $build[$key] = [
        '#type' => 'inline_template',
        '#template' => '<h4>{{ title }}</h4>{{ content }}<br />',
        '#context' => [
          'title' => $data[1],
          'content' => QtoolsDebug::export(QtoolsDebug::cloneFormVarDumper($data[0])),
        ],
      ];
    }

    return $build;
  }

}
